<?php
/* TOP_COMMENT_START
 * Copyright (C) 2022, Levion Partners dba PulseCMS - All Rights Reserved
 *
 * This file is part of Pulse Core. It may be used by individuals or organizations generating less than $400,000 USD per year in revenue, free-of-charge. Individuals or organizations generating over $400,000 in annual revenue who continue to use Pulse Core after 90 days for non-evaluation and non-development use must purchase a paid license. 
 *
 * Proprietary
 * You may modify this source code for internal use. Resale or redistribution is prohibited.
 *
 * You can get the latest version at: https://pulsecms.com/
 *
 * Dated June 2023
 *
TOP_COMMENT_END */

declare(strict_types = 1);

\error_reporting(\E_STRICT|\E_ALL);

require_once (__DIR__ . '/symlink_safe.php');

require_once (PULSE_BASE_DIR . "/config.php");
require_once (PULSE_BASE_DIR . "/{$admin}/inc/lang/english.php");

if (!empty($language) and \file_exists(PULSE_BASE_DIR . "/{$admin}/inc/lang/{$language}.php")) {
	require_once( PULSE_BASE_DIR . "/{$admin}/inc/lang/{$language}.php");
}

# \error_log( 'Load config + translation' ); # debug message

# ===========================================================================>
/**
 * wedge in the updated blog storage logic from pulsecore
 */
# start the session
\pulsecore\session\start();

# route requests as needed = NB get var changed here
$delta_get_vars = \pulsecore\dispatch( $_GET, $_POST, $_COOKIE, (isset($_SERVER['REQUEST_METHOD']) ? $_SERVER['REQUEST_METHOD'] : 'GET') );
$_GET = \array_merge( $_GET, $delta_get_vars );

# special case - rss blog page
if (isset($_GET['p']) and ($_GET['p'] == 'rss_blog')) {
	$pulsecore_page = new \pulsecore\page\rss\Blog();
	$pulsecore_page->process( $_GET, $_POST, $_COOKIE, (isset($_SERVER['REQUEST_METHOD']) ? $_SERVER['REQUEST_METHOD'] : 'GET') );
	exit;
}

# ===========================================================================>

# detect page to display
$page = (isset($_GET['p']) and !empty($_GET['p'])) ? $_GET['p'] : \pulsecore\wedge\config\get_json_configs()->json->home_page;
$page = htmlspecialchars($page, ENT_QUOTES, 'UTF-8');

# ==> begin wedge <==
# filter/clean incoming page parameter
$page = \pulsecore\filter\page( $page );

# check for cache entry - no caching for logged in users - no caching form posts
if (    !(\pulsecore\acl_role\is_administrator() or \pulsecore\acl_role\is_editor())
	   and (\pulsecore\wedge\config\get_json_configs()->json->cache_html_enable === true)
	   and (\sizeof($_POST) == 0)
	 ) {
	$cache_manager = new \pulsecore\cache\Manager();
	$cache_pool    = $cache_manager->pool( \pulsecore\cache\Manager::DAY_1 );
	
	$cache_item_name = "page-name-{$page}-" . \print_r($_GET, true);
	
	if ($cache_pool->is_valid($cache_item_name)) {
		echo $cache_pool->get($cache_item_name);
		exit;
	}
}

# meta tags
if (\pulsecore\wedge\config\get_json_configs()->json->theme_meta_author_show) {
	\pulsecore\get_context()->theme->meta->add( 'author', 'PulseCMS' );
}

# fitvids
\pulsecore\get_context()->theme->js_body->add( PULSE_BASE_URL . '/pulsecore/asset/vendor/node_modules/vanilla-fitvids/jquery.fitvids.js', [], 'fitvids' );
\pulsecore\get_context()->theme->js_body->add_inline(
	'fitvid-body',
<<<EOD
	jQuery(document).ready(
		function() {
			window.setTimeout(
				function() {
					jQuery("video").parent().fitVids();
					jQuery('iframe[src*="youtube"]').parent().fitVids();
				},
			);
		}
	);
EOD
	,
	['fitvids']
);

# load FontAwesome everywhere
if (\is_dir(PULSE_BASE_DIR . '/pulsecore/asset/vendor/fontawesome')) {
	\pulsecore\get_context()->theme->css->add( PULSE_BASE_URL . '/pulsecore/asset/vendor/fontawesome/css/all.min.css', [] );
} else {
	# \pulsecore\get_context()->theme->css->add( 'https://cdnjs.cloudflare.com/ajax/libs/font-awesome/5.11.2/css/all.min.css', [] );
}

# inline edit includes js/css
if (\pulsecore\acl_role\is_administrator() or \pulsecore\acl_role\is_editor() ) {
	
	$base_url = \pulsecore\wedge\config\get_json_configs()->json->path;
	
	# vue
	\pulsecore\get_context()->theme->js_body->add( PULSE_BASE_URL . '/pulsecore/asset/vendor/node_modules/vue/dist/vue.min.js', [], 'vue' );
	\pulsecore\get_context()->theme->js_body->add( PULSE_BASE_URL . '/pulsecore/asset/vendor/node_modules/vue-resource/dist/vue-resource.min.js', ['vue'] );
	
	# Redactor II
	\pulsecore\get_context()->theme->css->add( PULSE_ADMIN_URL . '/redactor/redactor.min.css',            [], 'redactor' );
	\pulsecore\get_context()->theme->css->add( PULSE_ADMIN_URL . '/redactor/plugins/clips.min.css',       ['redactor'] );
	\pulsecore\get_context()->theme->css->add( PULSE_ADMIN_URL . '/redactor/plugins/filemanager.min.css', ['redactor'] );
	\pulsecore\get_context()->theme->css->add( PULSE_ADMIN_URL . '/redactor/plugins/inlinestyle.min.css', ['redactor'] );
	#\pulsecore\get_context()->theme->css->add( PULSE_ADMIN_URL . '/codemirror/codemirror.css',            ['redactor'] );
	
	\pulsecore\get_context()->theme->js_body->add( PULSE_ADMIN_URL . '/redactor/redactor.min.js', [], 'redactor' );
	
	# Redactor Plugins - source.js is the old HTML viewer - replaced in favour to codemirror
	# \pulsecore\get_context()->theme->js_body->add( PULSE_ADMIN_URL . '/redactor/plugins/source.js', [PULSE_ADMIN_URL . '/redactor/redactor.min.js'] );
	
	# \pulsecore\get_context()->theme->js_body->add( PULSE_ADMIN_URL . '/codemirror/codemirror.min.js', ['redactor'] );
	# \pulsecore\get_context()->theme->js_body->add( PULSE_ADMIN_URL . '/codemirror/xml/xml.js',        ['redactor'] );
	
	\pulsecore\get_context()->theme->js_body->add( PULSE_ADMIN_URL . '/redactor/plugins/alignment.min.js',     ['redactor'] );
	\pulsecore\get_context()->theme->js_body->add( PULSE_ADMIN_URL . '/redactor/plugins/clips.min.js',         ['redactor'] );
	\pulsecore\get_context()->theme->js_body->add( PULSE_ADMIN_URL . '/redactor/plugins/filemanager.min.js',   ['redactor'] );
	\pulsecore\get_context()->theme->js_body->add( PULSE_ADMIN_URL . '/redactor/plugins/fontcolor.min.js',     ['redactor'] );
	\pulsecore\get_context()->theme->js_body->add( PULSE_ADMIN_URL . '/redactor/plugins/fontfamily.min.js',    ['redactor'] );
	\pulsecore\get_context()->theme->js_body->add( PULSE_ADMIN_URL . '/redactor/plugins/fontsize.min.js',      ['redactor'] );
	\pulsecore\get_context()->theme->js_body->add( PULSE_ADMIN_URL . '/redactor/plugins/fullscreen.min.js',    ['redactor'] );
	\pulsecore\get_context()->theme->js_body->add( PULSE_ADMIN_URL . '/redactor/plugins/imagemanager.min.js',  ['redactor'] );
	\pulsecore\get_context()->theme->js_body->add( PULSE_ADMIN_URL . '/redactor/plugins/inlinestyle.min.js',   ['redactor'] );
	\pulsecore\get_context()->theme->js_body->add( PULSE_ADMIN_URL . '/redactor/plugins/properties.min.js',    ['redactor'] );
	\pulsecore\get_context()->theme->js_body->add( PULSE_ADMIN_URL . '/redactor/plugins/table.min.js',         ['redactor'] );
	\pulsecore\get_context()->theme->js_body->add( PULSE_ADMIN_URL . '/redactor/plugins/textdirection.min.js', ['redactor'] );
	\pulsecore\get_context()->theme->js_body->add( PULSE_ADMIN_URL . '/redactor/plugins/video.min.js',         ['redactor'] );
	\pulsecore\get_context()->theme->js_body->add( PULSE_ADMIN_URL . '/redactor/plugins/widget.min.js',        ['redactor'] );
	
	\pulsecore\get_context()->theme->js_body->add( PULSE_ADMIN_URL . "/js/pulse_redactor_imagemanager.js",  ['redactor'] );
	
	# \pulsecore\get_context()->theme->js_body->add( PULSE_ADMIN_URL . '/redactor/plugins/codemirror.js',    ['redactor'] );
	# \pulsecore\get_context()->theme->js_body->add( PULSE_ADMIN_URL . '/redactor/plugins/snippets.js',      ['redactor'] );
	
	\pulsecore\get_context()->theme->js_body->add( PULSE_ADMIN_URL . '/redactor/mail.js',      ['redactor'] );
	
	$lang_code = \pulsecore\language_to_iso( $language );
	
	\pulsecore\get_context()->theme->js_body->add( PULSE_ADMIN_URL . "/redactor/lang/{$lang_code}.js", ['redactor'] );
	
	\pulsecore\get_context()->theme->js_body->add( PULSE_ADMIN_URL . "/redactor/lang.pulse.js", ['redactor'] );
	
	$sweetalert_active  = \pulsecore\wedge\config\get_json_configs()->json->sweetalert->active;
	$sweetalert_timeout = \pulsecore\wedge\config\get_json_configs()->json->sweetalert->timeout;
	
	$sweetalert_active  = \intval( $sweetalert_active );
	$sweetalert_timeout = \intval( $sweetalert_timeout );
	
	$js_redactor_inline_lang =<<<EOD
window.pulsecore = window.pulsecore || {};
pulsecore.alert = {
	active:  {$sweetalert_active},
	timeout: {$sweetalert_timeout}
};
window.pulsecore.lang       = "{$language}";
window.pulsecore.lang_short = "{$lang_code}";
EOD;
	
	\pulsecore\get_context()->theme->js_body->add_inline( 'redactor-inline-lang', $js_redactor_inline_lang );
	
	\pulsecore\get_context()->theme->js_body->add_inline( 'translations',         \pulsecore\get_context()->theme->translations->render() );
	
	#Redactor Initialisation on #wysiwyg -->
	#\pulsecore\get_context()->theme->js_body->add( PULSE_ADMIN_URL . '/redactor/redactor_init.js', [] );
}

# inject page level css/js from settings
if (\strlen(\pulsecore\wedge\config\get_json_configs()->json->inline->css) > 0) {
	\pulsecore\get_context()->theme->css->add_inline(     'page_css', \pulsecore\wedge\config\get_json_configs()->json->inline->css );
}
if (\strlen(\pulsecore\wedge\config\get_json_configs()->json->inline->js) > 0) {
	\pulsecore\get_context()->theme->js_body->add_inline( 'page_js',  \pulsecore\wedge\config\get_json_configs()->json->inline->js  );
}
#==> end wedge   <==

# load home.txt for folders
if (preg_match("/\//", $page)) {
	if (\file_exists("content/pages/".$page."home.txt")) {
		$page = $page . "home";
	}
}

# 404s
if (!file_exists("content/pages/".$page.".txt")) {
	$page = '404';
	header('HTTP/1.1 404 Not Found');
}

# set the CORS header
\header( 'Access-Control-Allow-Origin: *' ); # CORS

# disable parsedown for inline edit mode 
if (!\pulsecore\acl_role\is_administrator() and !\pulsecore\acl_role\is_editor()) {
	require_once (PULSE_BASE_DIR . '/inc/plugins/parsedown.php');
	$parsedown = new \Parsedown();
	
	\pulsecore\get_context()->parsedown = $parsedown;
}

# parse the page storage
$page_datum = new \pulsecore\store\page\Item();
$page_datum->load( "content/pages/{$page}.txt" );

$page_title   = $page_datum->title;
$page_desc    = $page_datum->description;
$content      = $page_datum->html;
$new_template = $page_datum->page_template;

# access control for user groups and logged in members
if (\pulsecore\acl_role\user_group_test_resource_controlled("pages/{$page}")) {
	
	$allow_access = false;
	
	# admins and editors always have access
	$allow_access = ($allow_access or \pulsecore\acl_role\is_administrator() or \pulsecore\acl_role\is_editor());
	
	# check logged in user permissions
	if ((!$allow_access) and \pulsecore\acl_role\test_logged_in()) {
		$allow_access = ($allow_access or \pulsecore\acl_role\user_group_test_user_allowed("pages/{$page}", 'r', $_SESSION['user_group_list']));
	}
	
	# skip page if no access allowed
	if (!$allow_access) {
		\pulsecore\session\status_add( $GLOBALS['lang_settings_user_group_list_access_denied'] );
		
		\header( "Location: {$GLOBALS['path']}/" );
		exit;
	}
}

# inline edit
if (\pulsecore\acl_role\can_edit_resource( \pulsecore\wedge\config\get_json_configs()->json->navigation_options->logged_in_menu )) {
	
	# widget - inline edit
	\pulsecore\get_context()->theme->css->add( PULSE_BASE_URL . '/pulsecore/asset/js/widget/inline-edit/inline-edit.css', [] );
	
	\pulsecore\get_context()->theme->js_body->add_inline( 'page_js', "window.pulsecore = window.pulsecore || {}; window.pulsecore.base_url = \"" . PULSE_BASE_URL . "\"; window.pulsecore.admin_url = \"" . PULSE_ADMIN_URL . "\";window.pulsecore.inline_edit = {app: {}, component: {}};" );
	\pulsecore\get_context()->theme->js_body->add( PULSE_BASE_URL . '/pulsecore/asset/js/widget/inline-edit/message-bus.js', ['vue', 'redactor'] );
	\pulsecore\get_context()->theme->js_body->add( PULSE_BASE_URL . '/pulsecore/asset/js/widget/inline-edit/content.js',     ['vue', 'redactor'] );
	\pulsecore\get_context()->theme->js_body->add( PULSE_BASE_URL . '/pulsecore/asset/js/widget/inline-edit/global-save.js', ['vue', 'redactor'] );
	\pulsecore\get_context()->theme->js_body->add( PULSE_BASE_URL . '/pulsecore/asset/js/widget/inline-edit/modal.js',       ['vue', 'redactor'] );
	\pulsecore\get_context()->theme->js_body->add( PULSE_BASE_URL . '/pulsecore/asset/js/widget/inline-edit/toolbar.js',     ['vue', 'redactor'] );
	\pulsecore\get_context()->theme->js_body->add( PULSE_BASE_URL . '/pulsecore/asset/js/widget/inline-edit/inline-edit.js', ['vue', 'redactor'], 'inline-edit' );
	
	$view_helper_inline_edit = new \pulsecore\view\helper\InlineEdit();
	$content = $view_helper_inline_edit->render( [$page, 'page', $content] );
}

# meta
if (\strlen($page_datum->meta_custom_description) > 0) {
	#\pulsecore\get_context()->theme->meta->add( 'custom_meta', $page_datum->meta_custom_description );
	\pulsecore\get_context()->theme->meta->add( 'description', $page_datum->meta_custom_description );
} else {
	\pulsecore\get_context()->theme->meta->add( 'description', $page_datum->description );
}

$meta_robots   = [];
$meta_robots[] = ($page_datum->meta_indexed   == 'yes') ? 'index'   : 'noindex';
$meta_robots[] = ($page_datum->meta_no_follow == 'yes') ? 'nofollow': 'follow';

$meta_robots = \implode(',', $meta_robots ); 

\pulsecore\get_context()->theme->meta->add( 'robots', $meta_robots );

# inject page data into context storage
\pulsecore\get_context()->state->page = new \stdClass();
\pulsecore\get_context()->state->page->page_title     = (isset($page_title) ? $page_title : '');
\pulsecore\get_context()->state->page->page_desc      = (isset($page_desc ) ? $page_desc  : '');
\pulsecore\get_context()->state->page->page_body      = (isset($content   ) ? $content    : '');
\pulsecore\get_context()->state->page->path           = (isset($path      ) ? $path       : '/');
\pulsecore\get_context()->state->page->location       = $page_datum->get_location();
\pulsecore\get_context()->state->page->created_on     = $page_datum->created_on;
\pulsecore\get_context()->state->page->modified_on    = $page_datum->modified_on;
\pulsecore\get_context()->state->page->page_info_url  = \pulsecore\page_info_url($_SERVER['REQUEST_URI'], \pulsecore\wedge\config\get_json_configs()->json->url_prefix);
\pulsecore\get_context()->state->page->page_info_blog = \pulsecore\page_info_blog( \pulsecore\get_context()->state->page->page_info_url, \pulsecore\wedge\config\get_json_configs()->json->url_prefix );

# page inline css/js
if (\strlen($page_datum->inline_css) > 0) {
	\pulsecore\get_context()->theme->css->add_inline(     'per_page_css', $page_datum->inline_css );
}
if (\strlen($page_datum->inline_js) > 0) {
	\pulsecore\get_context()->theme->js_body->add_inline( 'per_page_js',  $page_datum->inline_js );
}

# page language setting
if (\strlen($page_datum->meta_language) > 0) {
	\pulsecore\get_context()->theme->language->set($page_datum->meta_language);
}


$template_file_path     = '';
$theme_full_folder_path = '';
$template_folder        = '';

$chosenTheme = '';
// $template_file_path is the full path to the layout.php file.
// $template_folder is the full path to the template, ex: /

if (!empty($new_template) and \file_exists("template/{$new_template}/layout.php")) { 
	# Case 1: Specified a custom, non-default theme.
	$chosenTheme = $new_template;
	
/* Commenting this out. This logic would always send someone to the /template/default theme, 
whereas in v5.4 we want to send them to their chosen default. Not the theme called "default".
-SL

} else if (!empty($new_template) and ($new_template == 'default') and \file_exists("template//layout.php")) { 
	# Case 2: The "default" template was explicitely specified in the page settings.
    $chosenTheme='default';
*/

} else { 
	# Case 3: We do not have any template specified. (Blog default? Page has no specific template listed?)
	# In this case, we choose the default from settings.

	# It looks like this code is a fallback- perhaps if we're using JSON storage.
	# It checks if there's a value for the selected theme in json storage, and if not, it goes to the default.
	$default_theme_selected = (isset(\pulsecore\wedge\config\get_json_configs()->json->theme_selected) ? \pulsecore\wedge\config\get_json_configs()->json->theme_selected : 'default');
	$chosenTheme = $default_theme_selected;
}

list($template_file_path, $template_folder) = \pulsecore\theme\load_theme($chosenTheme, $path);
// In the line above, load_theme returns an array with 2 items.
// PHP's list() function puts each of these into the right variables.
// It's a cute way to get multiple values from a function in PHP.
// $template_file_path is the path to the layout.php file.
// $template_folder is the path to the template, ex: /pulsecms5/template/my_template

\pulsecore\get_context()->state->chosenTheme        = $chosenTheme;
\pulsecore\get_context()->state->template_file_path = $template_file_path;
\pulsecore\get_context()->state->template_folder    = $template_folder;

# expand page content to get the list of css/js resources to load
$content = \pulsecore\tag_runner\expand( $content );

\pulsecore\get_context()->content = $content; # used for the main_content tag so not the complete page

/*
if (preg_match_all("/".'(\\{)'.'(\\{)'.'.*?'.'(\\})'.'(\\})'."/", $content, $m)) {
    
			foreach ($m[0] as $get_embed1) {
				
				# reset the tag variables
				$tag_var1 = '';
				$tag_var2 = '';
				$tag_var3 = '';
				
        $get_embed = $get_embed1;
        $get_embed = str_replace("{", "" ,$get_embed); 
        $get_embed = str_replace("}", "" ,$get_embed);  
                  
        if (substr_count($get_embed, ':') >=1 ) {                        
            $exp = explode(':', $get_embed); 
            $vars = array_slice($exp, 1); 
            $tag_var1 = (!empty($vars[0])) ? $vars[0] : '';
            $tag_var2 = (!empty($vars[1])) ? $vars[1] : '';
            $tag_var3 = (!empty($vars[2])) ? $vars[2] : '';
            $tag_var4 = (!empty($vars[3])) ? $vars[3] : '';
            $tag_var5 = (!empty($vars[4])) ? $vars[4] : '';
            $tag_var6 = (!empty($vars[5])) ? $vars[5] : '';
            $tag_var7 = (!empty($vars[6])) ? $vars[6] : '';
            $get_embed = $exp[0];
        }
        
        #replace a block with its content
        \ob_start(); 
        if ($get_embed == 'template') {
        	$new_template = $tag_var1;
        	$new          = '';
        } else {
        	#filter
        	$get_embed = \pulsecore\filter\item_url( $get_embed );
        	
        	include (PULSE_BASE_DIR . "/inc/tags/{$get_embed}.php");
        	$new = \ob_get_contents();
        }
        $content = \str_replace($get_embed1, $new, $content);
        \ob_end_clean();
    }
}
*/

\ob_start();
// Keyword. Here is where we load the theme.
// @logic We get the path to the theme file and load the theme in this area.
// $new_template is like "main" or "one-page-theme", etc. Just a name.
// Simon added Nov 20, 2020: Get theme path.


/*
@logic Adding a section to debug output as it goes across the processing.
From Simon:
While I'm new to the code, I do not see any classes for debugging.
This should be improved upon for deeper code debugging.

The folder $debugOutputPath will contain the debug files.

Remember to include the trailing "/"
Ex: $debugOutputPath = '/users/my_user/';
or (Windows)
Ex: $debugOutputPath = 'C:\myfolder\';
or save to Document root for web hosts:
Ex: $debugOutputPath = $_SERVER['DOCUMENT_ROOT'].'/';

Set $pulseDebugLevel = 1 or higher to generate the output.

While traditional debugging is preferred, having these full files
generated for inspection without fumbling through a debugger Watch window
is helpful.
*/
$debugOutputPath = $_SERVER['DOCUMENT_ROOT'].'/';
$pulseDebugLevel = 0;

if ($pulseDebugLevel > 0) {
	file_put_contents ($debugOutputPath.'content-01-pre-theme.html' , $content);
}

include ($template_file_path);

$content = \ob_get_contents();
\ob_end_clean();

if ($pulseDebugLevel > 0) {
	file_put_contents ($debugOutputPath.'content-02-post-theme.html' , $content);
}

# expand the layout tags
$content = \pulsecore\tag_runner\expand( $content );

# inject google analytics into the html
if (\strlen(\trim(\pulsecore\wedge\config\get_json_configs()->json->google_analytics)) > 0) {
	$content = \str_replace('<!-- GOOGLE ANALYTICS -->', \pulsecore\wedge\config\get_json_configs()->json->google_analytics, $content);
}

# inject OGP content into the html
/*
include_once (PULSE_BASE_DIR . '/inc/plugins/ogp.php');
\ob_start();
ogp_render();
$capture_ogp = \ob_get_contents();
\ob_end_clean();
if (\strlen(\trim($capture_ogp)) > 0) {
	$content = \str_replace('<!-- OGP PLACEHOLDER -->', $capture_ogp, $content);
}
*/
$content = \str_replace(
	'<!-- OGP PLACEHOLDER -->',
	\pulsecore\tags\SocialExposure::execute_tag( [], [] ),
	$content
);

if ($pulseDebugLevel > 0) {
	file_put_contents ($debugOutputPath.'/content-03-post-tags.html' , $content);
}

# save in cache
if (   !(\pulsecore\acl_role\is_administrator() or \pulsecore\acl_role\is_editor())
	and (\pulsecore\wedge\config\get_json_configs()->json->cache_html_enable === true)
	and (\sizeof($_POST) == 0)
	) {
	$cache_pool->set($cache_item_name, $content, ['page'] );
}

# write content to stdout
# \error_log( "write html" ); # debug message
# \error_log( print_r(headers_list(), true) );
# \error_log( $content ); # debug message

echo $content;
exit;
