
/**
 * imports
 */
import message_bus from '../message-bus.js';
import storage     from '../storage.js';

import Menu     from '../model/menu.js';
import MenuItem from '../model/menu_item.js';

// start up
const menu_list = {
	
	/**
	 * state
	 */
	data: function () {
		return {
			
			data_list: []
		}
	},
	
	/**
	 *
	 */
	methods: {
		
		/**
		 * check that menu item is a MenuItem
		 * @param mixed item
		 * @return boolean
		 */
		is_menu_item_type: function (item) {
			return (item instanceof MenuItem);
		},
		
		/**
		 * check that menu is on the open menu list
		 * @param mixed item
		 * @return boolean
		 */
		is_menu_open: function (item) {
			
			/*
			const result =
				this.is_menu_type(item) &&
				(this.open_menus.indexOf(item) > 0);
			*/
			
			// console.log( 'is_menu_open', item.name, item, this.open_menus );
			
			let result = false;
			
			if (this.is_menu_type(item)) {
				
				for (let k = 0; k < this.open_menus.length; k++) {
					
					const tmp = this.open_menus[k];
					
					// console.log( item.name, tmp.name );
					
					if (item.equals(tmp)) {
						result = true;
						break;
					}
				}
			}
			
			return result;
		},
		
		/**
		 * check that menu item is a Menu
		 * @param mixed item
		 * @return boolean
		 */
		is_menu_type: function (item) {
			return (item instanceof Menu);
		},
		
		/**
		 * toggle activation
		 */
		onclick_active: function (item) {
			
			message_bus.$emit( 'menu-activate-toggle', item );
		},
		
		/**
		 * delete a menu item
		 */
		onclick_nuke: function (item) {
			
			message_bus.$emit( 'menu-delete-toggle', item );
		},
		
		/**
		 * delete a menu item
		 */
		onclick_menu_item_delete: function (item) {
			
			message_bus.$emit( 'menu-delete', item );
		},
		
		/**
		 * DnD done
		 */
		on_dnd_change_menu: function (arg) {
			
			message_bus.$emit( 'dnd-change', this.menu );
		}
	},
	
	/**
	 * props
	 */
	props: [
		'menu',
		'open_menus',
		'parent'
	],
	
	/**
	 * html
	 */
	template: `
<!-- start menu-list -->
<div class="menu-list">

		<draggable class="draggble" v-model="menu.menu_items" :options="{draggable:'.menu_box', group: 'menu'}" v-on:change="on_dnd_change_menu">
		
			<div v-for="item in menu.menu_items" :key="item.name" class="menu_box">
			
				<template v-if="is_menu_type(item)">
					
					<span class="delete" v-on:click="onclick_menu_item_delete(item)" v-show="item.name != 'pending'">
						<i class="fa fa-trash"></i>
					</span>
					
					<label v-show="item.name != 'pending'">
						{{ 'lang_settings_navigation_activate' | translate }}
						<input class="active" type="checkbox" v-model="item.active" v-on:click="onclick_active(item)" />
					</label>
					<label v-show="item.name != 'pending'">
						{{ 'lang_del_button' | translate }}:
						<input class="nuke" type="checkbox" v-model="item.nuke"  v-on:click="onclick_nuke(item)" />
					</label>
					
					{{item.name}}
					
					<expander type="toggle" v-bind:control_item="item" v-bind:open="is_menu_open(item)">
						<menu-list v-bind:menu="item" v-bind:open_menus="open_menus" v-bind:parent="menu"></menu-list>
					</expander>
					
					<footer>
						<expander type="arrow" v-bind:control_item="{}" v-bind:open="false">
							<add-menu v-bind:parent="item"></add-menu>
						</expander>
						
						<save-button></save-button>
						
						<non-pulse-button v-bind:parent="item"></non-pulse-button>
					</footer>
				</template>
				
				<template v-if="is_menu_item_type(item)">
					<menu-item v-bind:menu="menu" v-bind:item="item">
					</menu-item>
				</template>
			</div>
		</draggable>
	
</div>
<!-- end menu-list -->
`,

	/**
	 * watched properties
	 */
	watch: {
		
		//deep: true,
		//immediate: true,
		
		menu: function (new_val, old_val) {
			
			this.data_list = new_val.menu_items;
		}
	}
};

/**
 * exports
 */
export default menu_list;

