
/**
 * imports
 */
import message_bus from './message-bus.js';
import storage     from './storage.js';
import track_ui    from './track_ui.js';

import component_add_menu          from './component/add-menu.js';
import component_expander          from './component/expander.js';
import component_manage_navigation from './component/manage-navigation.js';
import component_menu_item         from './component/menu-item.js';
import component_menu_list         from './component/menu-list.js';
import component_non_pulse_button  from './component/non-pulse-button.js';
import component_save_button       from './component/save-button.js';

import Menu     from './model/menu.js';
import MenuItem from './model/menu_item.js';

/**
 * state management
 */
const state = {
	menu: storage.get_root(),
	
	open_menus: []
};

/**
 * add components
 */
Vue.component( 'add-menu',         component_add_menu );
Vue.component( 'expander',         component_expander );
Vue.component( 'menu-item',        component_menu_item );
Vue.component( 'menu-list',        component_menu_list );
Vue.component( 'non-pulse-button', component_non_pulse_button );
Vue.component( 'save-button',      component_save_button );

/**
 * translate a string - if possible
 * @param string arg
 * @return string
 */
Vue.filter(
	'translate',
	function (arg) {
		
		const result = window.pulsecore._(arg);
		
		return result;
	}
);

// start up
const app = new Vue(
	{
		el:   '.pulsecore.manage-navigation',
		
		/**
		 * components used in this widget/component
		 */
		components: {
			'manage-navigation': component_manage_navigation
		},
		
		/**
		 * state
		 */
		data: state,
		
		/**
		 * filters
		 */
		filters: {
		},
		
		/**
		 * behaviour
		 */
		methods: {
			
			
		},
		
		/**
		 * life cycle hook
		 */
		mounted: function () {
			
			const self = this;
			
			/*
			 * custom event - storage change
			 */
			message_bus.$on(
				'storage-change',
				function (root) {
					
					Vue.set( self, 'menu', new Menu('dummy', '', false, false, []) );
					
					self.$nextTick(
						function () {
							Vue.set( self, 'menu', root );
						}
					);
				}
			);
			
			/*
			 * custom event - open menu list change
			 */
			message_bus.$on(
				'track-ui-menu-change',
				function (open_menu_list) {
					
					console.log( 'open_menu_list', JSON.parse( JSON.stringify(open_menu_list) ) );
					
					Vue.set( self, 'open_menus', [] );
					
					self.$nextTick(
						function () {
							Vue.set( self, 'open_menus', open_menu_list );
						}
					);
				}
			);
		}
	}
);




/**
 * exports
 */
window.pulsecore.manage_navigation.app = app;

export default app;

