/**
 * imports
 */
import message_bus from './message-bus.js';

import Menu     from './model/menu.js';
import MenuItem from './model/menu_item.js';

/**
 * data storage
 */
const storage = {
	
	/**
	 * state storage
	 */
	root: false,
	
	/**
	 * event handlers
	 */
	event: {
		/**
		 * attach all the events we will use
		 */
		attach: function () {
			
			message_bus.$on( 'dnd-change', this.dnd_change);
			
			message_bus.$on( 'menu-add',  this.menu_add );
			
			message_bus.$on( 'menu-add-non-pulse',  this.menu_add_non_pulse );
			
			message_bus.$on( 'menu-delete', this.menu_delete );
			
			message_bus.$on( 'menu-activate-toggle', this.menu_activate_toggle);
			
			message_bus.$on( 'menu-delete-toggle', this.menu_delete_toggle );
			
			message_bus.$on( 'menu-item-delete', this.menu_item_delete);
			
			message_bus.$on( 'menu-item-activate-toggle', this.menu_item_activate_toggle);

			message_bus.$on( 'menu-item-delete-toggle', this.menu_item_delete_toggle);
			
			message_bus.$on( 'state-save', this.save_state);
		},
		
		/**
		 * handle event
		 */
		dnd_change: function(item) {
			
			const path = [];
			
			const probe = storage.find( storage.get_root(), item );
			
			for (let k = 0; k < probe.path.length; k++) {
				
				const tmp = probe.path[k];
				
				if (tmp.name != 'ROOT') {
					path.push( tmp.name );
				}
			}
			
			//console.log( path );
			
			storage.set_sub_menus(
				path,
				item
			);
			
			//console.log( storage.get_root().pretty_print() );
			
			message_bus.$emit( 'storage-change', storage.get_root() );
		},
		
		/**
		 * handle event
		 */
		menu_add: function(parent, name) {
			
			const path = [];
			
			//console.log( parent, name );
			
			const probe = storage.find( storage.get_root(), parent );
			
			for (let k = 0; k < probe.path.length; k++) {
				
				if (probe.path[k].name != 'ROOT') {
					path.push( probe.path[k].name );
				}
			}
			
			path.push( name );
			
			//console.log( path );
			
			
			/*
			// corner case - root menu
			if (parent == false) {
				
				path.unshift( 'ROOT' );
			}
			*/
			storage.set_sub_menus(
				path,
				new Menu( name, '', true, false, [] )
			);
			
			message_bus.$emit( 'storage-change', storage.get_root() );
		},
		
		/**
		 * handle event
		 */
		menu_add_non_pulse: function(parent, name, url) {
			
			const path = [];
			
			//console.log( parent, name );
			
			const probe = storage.find( storage.get_root(), parent );
			
			for (let k = 0; k < probe.path.length; k++) {
				
				if (probe.path[k].name != 'ROOT') {
					path.push( probe.path[k].name );
				}
			}
			
			path.push( name );
			
			//console.log( path );
			
			
			/*
			// corner case - root menu
			if (parent == false) {
				
				path.unshift( 'ROOT' );
			}
			*/
			storage.set_sub_menus(
				path,
				new MenuItem( name, url, true, false, [] )
			);
			
			message_bus.$emit( 'storage-change', storage.get_root() );
		},
		
		/**
		 * handle event
		 */
		menu_delete: function (item) {
			
			storage.remove( storage.get_root(), item );
			
			message_bus.$emit( 'storage-change', storage.get_root() );
		},
		
		/**
		 * handle event
		 */
		menu_activate_toggle: function (item) {
			
			const probe = storage.find( storage.get_root(), item );
			
			if (probe.path.length > 0) {
				const a = probe.path.pop();
				
				a.set_active( !a.active );
			}
			
			message_bus.$emit( 'storage-change', storage.get_root() );
		},
		
		/**
		 * handle event
		 */
		menu_delete_toggle: function (item) {
			
			//console.log( item.name );
			
			const probe = storage.find( storage.get_root(), item );
			
			//console.log( probe, item.name );
			
			if (probe.path.length > 0) {
				const a = probe.path.pop();
				
				a.set_nuked( !a.nuke );
			}
			
			message_bus.$emit( 'storage-change', storage.get_root() );
		},
		
		/**
		 * handle event
		 */
		menu_item_delete: function (parent, item) {
			
			storage.remove( storage.get_root(), item );
			
			message_bus.$emit( 'storage-change', storage.get_root() );
		},
		
		/**
		 * handle event
		 */
		menu_item_activate_toggle: function (parent, item) {
			
			const probe = storage.find( storage.get_root(), item );
			
			if (probe.path.length > 0) {
				const a = probe.path.pop();
				
				a.set_active( !a.active );
			}
			
			message_bus.$emit( 'storage-change', storage.get_root() );
		},
		
		/**
		 * handle event
		 */
		menu_item_delete_toggle: function (parent, item) {
			
			const probe = storage.find( storage.get_root(), item );
			
			if (probe.path.length > 0) {
				const a = probe.path.pop();
				
				a.set_nuked( !a.nuke );
			}
			
			message_bus.$emit( 'storage-change', storage.get_root() );
		},
		
		/**
		 * handle event
		 */
		save_state: function () {
			
			storage.save_state();
		}
	},
	
	/**
	 * find a menu/menu-item
	 * @param Menu menu the menu to search on
	 * @param object thingy the object to search for
	 * @return object of the parent, grand parents etc of the object found
	 */
	find: function (menu, thingy) {
		
		const result = {found: false, path: []};
		
		if (menu == thingy) {
			
			result.found = true;
			result.path.push( menu );
			
		} else {
			
			if (menu instanceof Menu) {
				
				for (let p = 0; p < menu.menu_items.length; p++) {
					
					const item = menu.menu_items[p];
					
					const probe = this.find( item, thingy);
					
					if (probe.found == true) {
						
						result.found = true;
						result.path  = [menu].concat( probe.path );
						
						break;
					}
				}
			}
		}
		
		return result;
	},
	
	/**
	 * get full menu
	 * @return object
	 */
	get_root: function () {
		
		const root = this.root;
		
		// ensure that top level "all" name is present
		let detected = root.has_menu('all');
		
		if (detected == false) {
			
			root.add(
				new Menu( 'all', 'all', true, false, [] )
			);
		}
		
		// ensure that top level "pending" name is present
		detected = root.has_menu('pending');
		
		if (detected == false) {
			
			root.add(
				new Menu( 'pending', 'pending', true, false, [] )
			);
		}
		
		return root;
	},
	
	/**
	 * import the server state and massage into something the JS can use
	 * @param string input_state
	 * @return array
	 */
	import_server_state: function (input_state) {
		
		const result = [];
		
		for (let k in input_state) {
			
			const item = input_state[k];
			
			// menu
			if ((typeof item.active == 'undefined') && (typeof item.url == 'undefined')) {
				
				const content = this.import_server_state( item );
				
				const tmp = new Menu( k, '', true, false, content );
				
				result.push( tmp );
			}
			
			// menu item
			if ((typeof item.active == 'boolean') && (typeof item.url == 'string')) {
				
				const tmp = new MenuItem( k, item.url, item.active, false );
				
				result.push( tmp );
			}
		}
		
		return result;
	},
	
	/**
	 * merge menu items into one 
	 * @param array of MenuItems
	 * @param string name
	 * @return array
	 */
	merge_array: function (menu_items, name) {
		
		const result = new Menu( name, '', true, false, [] );
		
		for (let k = 0; k < menu_items.length; k++) {
			
			const item = menu_items[k];
			
			result.menu_items.push( item );
		}
		
		return result;
	},
	
	/**
	 * remove an item from menu
	 * @param Menu menu the menu to search on
	 * @param object thingy the object to search for
	 * @return void
	 */
	remove: function (menu, thingy) {
		
		// safety - cant delete all menu
		if (thingy.name == 'all') {
			return;
		}
		
		// safety - cant delete pending menu
		if (thingy.name == 'pending') {
			return;
		}
		
		// process delete
		const probe = storage.find( menu, thingy );
		
		const removed_list = [];
		
		if (probe.path.length > 1) {
			
			const a = probe.path.pop();
			const b = probe.path.pop();
			
			//console.log( 'a', a.pretty_print() );
			//console.log( 'b', b.pretty_print() );
			
			const tmp = [];
			
			for (let k = 0; k < b.menu_items.length; k++) {
				
				const item = b.menu_items[k];
				
				if (item != a) {
					tmp.push( item );
				} else {
					removed_list.push( item );
				}
			}
			
			b.menu_items = tmp;
		}
		
		//console.log( menu.pretty_print() );
		
		// next step - moved deleted items onto the pending list
		const pending_menu = this.get_root().find_by_name('pending');
		
		//console.log( 'pending_menu', pending_menu );
		
		while (removed_list.length > 0) {
			
			const qqq = removed_list.pop();
			
			if (qqq instanceof MenuItem) {
				pending_menu.add( qqq );
			}
			
			if (qqq instanceof Menu) {
				
				for (let k = 0; k < qqq.menu_items.length; k++) {
					pending_menu.push( qqq.menu_items[k] );
				}
			}
		}
	},
	
	/**
	 * transmit the form
	 */
	save_state: function () {
		
		console.log( this.root.pretty_print() );
		
		// consolidate
		const data = {
			active:   this.root.build_active_list(),
			nav_list: this.root.build_nav_list(),
			nuke:     this.root.build_nuke_list(),
			
			csrf_token: pulsecore.manage_navigation.csrf_token
		};
		
		// now post
		const deferred = jQuery.post(
			pulsecore.admin_url + '/index.php?p=manage_navigation&method=put',
			data
		);
		
		deferred.done(
			function () {
				window.location.href = (pulsecore.admin_url + '/index.php?p=manage_navigation');
			}
		);
	},
	
	/**
	 * set sub menus for menu
	 * @param array menu For a single menu use ['a']  for deep menus use ['a', 'b', 'c']
	 * @param object arg the new sub menu
	 * @return void
	 */
	set_sub_menus: function (menu, arg) {
		
		let root = this.root;
		
		for (let k = 0; k < menu.length; k++) {
			
			const index = menu[k];
			
			let found = false;
			
			// find menu item
			for (let p = 0; p < root.menu_items.length; p++) {
				
				const item = root.menu_items[p];
				
				if ((item.name == index) && (item instanceof Menu)) {
					
					root = root.menu_items[p];
					
					found = true;
				}
			}
			
			// no menu item
			if (found == false) {
				
				root.menu_items.push( arg );
			}
		}
	}
};

/**
 * INITIALISE
 */
storage.root = storage.merge_array(
	storage.import_server_state( pulsecore.manage_navigation.data ),
	'ROOT'
);

// wire on events
storage.event.attach();




/**
 * exports
 */
window.pulsecore.manage_navigation.component.storage = storage;

export default storage;
