<?php
/* TOP_COMMENT_START
 * Copyright (C) 2022, Levion Partners dba PulseCMS - All Rights Reserved
 *
 * This file is part of Pulse Core. It may be used by individuals or organizations generating less than $400,000 USD per year in revenue, free-of-charge. Individuals or organizations generating over $400,000 in annual revenue who continue to use Pulse Core after 90 days for non-evaluation and non-development use must purchase a paid license. 
 *
 * Proprietary
 * You may modify this source code for internal use. Resale or redistribution is prohibited.
 *
 * You can get the latest version at: https://pulsecms.com/
 *
 * Dated June 2023
 *
TOP_COMMENT_END */

declare(strict_types = 1);

namespace pulsecore;

/**
 * generate a new secret
 * @param string $acl_role
 * @return string
 */
function otp_generate_secret (string $acl_role) : string {

	$totp = otp_get( $acl_role );

	$result = $totp->getSecret();

	return $result;
}

/**
 * basic OTP interface
 * @param string $acl_role
 * @return \OTPHP\TOTP
 */
function otp_get (string $acl_role) : \OTPHP\TOTP {
	
	\pulsecore\pre_condition(      isset($acl_role) );
	\pulsecore\pre_condition( \is_string($acl_role) );
	\pulsecore\pre_condition(    \strlen($acl_role) > 0);
	
	static $storage = [];
	
	$totp = false;
	
	if ($totp === false) {
		#new totp
		$totp = \OTPHP\TOTP::create(
			\pulsecore\wedge\config\get_json_configs()->json->otp_shared_secret,
			\pulsecore\get_configs()->otp->interval,
			\pulsecore\get_configs()->otp->digest,
			\pulsecore\get_configs()->otp->digits
		);
		$totp->setLabel( \pulsecore\get_configs()->otp->label);
			
		#editors
		if ($acl_role == \pulsecore\get_configs()->acl_role->editor) {

			$totp = \OTPHP\TOTP::create(
				\pulsecore\wedge\config\get_json_configs()->json->editor_user_otp_shared_secret,
				\pulsecore\get_configs()->otp->interval,
				\pulsecore\get_configs()->otp->digest,
				\pulsecore\get_configs()->otp->digits
			);
			$totp->setLabel( \pulsecore\get_configs()->otp->label );
		}
			
		$storage[$acl_role] = $totp;
		
	} else {
		#existing totp
		$totp = $storage[$acl_role];
	}
	
	return $totp;
}

/**
 * verify OTP password
 * @param string $otp_password
 * @param string $acl_role
 * @return bool
 */
function otp_verify_password (string $otp_password, string $acl_role) : bool {
	
	$totp = otp_get( $acl_role );
	
	$result = $totp->verify( $otp_password );
	
	return $result;
}
