<?php
/* TOP_COMMENT_START
 * Copyright (C) 2022, Levion Partners dba PulseCMS - All Rights Reserved
 *
 * This file is part of Pulse Core. It may be used by individuals or organizations generating less than $400,000 USD per year in revenue, free-of-charge. Individuals or organizations generating over $400,000 in annual revenue who continue to use Pulse Core after 90 days for non-evaluation and non-development use must purchase a paid license. 
 *
 * Proprietary
 * You may modify this source code for internal use. Resale or redistribution is prohibited.
 *
 * You can get the latest version at: https://pulsecms.com/
 *
 * Dated June 2023
 *
TOP_COMMENT_END */

namespace pulsecore\page\admin;

/**
 * manage navigation bar
 */
class ManageNavigation extends Base {
	
	/**
	 * get request
	 * @param array $request_params array of request parameters
	 * @param array $request_cookie array of cookie parameters
	 * @return string
	 */
	protected function handle_get (array $request_params, array $request_cookie) : string {
		
		# render model
		$view_model = new \pulsecore\ViewModel();
		
		# build navigation list
		$navigation = \pulsecore\get_configs()->navigation;
		
		# load stored version - if present
		if (isset(\pulsecore\wedge\config\get_json_configs()->json->navigation)) {
			$navigation = \pulsecore\wedge\config\get_json_configs()->json->navigation;
		}
		
		$view_model->navigation = $navigation;
		
		# pages not in the navigation list  ie new pages
		$page_list = \pulsecore\generate_non_navigation_pages( \pulsecore\wedge\config\get_json_configs()->json->path, \pulsecore\wedge\config\get_json_configs()->json->navigation);
		
		# merge the lists NB the pending menu
		if (!isset($view_model->navigation->pending)) {
			$view_model->navigation->pending = new \stdClass();
		}
		
		$view_model->navigation->pending = (object)\array_merge(
			$page_list,
			((array)$view_model->navigation->pending)
		);
		
		$view_model->navigation_packed = \json_encode($view_model->navigation);
		
		# error trapping
		if (!\is_string($view_model->navigation_packed)) {
			
			$view_model->navigation_packed = '{all: {}, pending: {}}';
			
			\error_log( 'pulsecore/page/admin/manage_navigation.php: Unable to build navigation list START' );
			\error_log( 'JSON error code: ' . \json_last_error() );
			\error_log( 'page list: '  . print_r($page_list, true) );
			\error_log( 'navigation: ' . print_r($navigation, true) );
			\error_log( 'combined: '   . print_r($view_model->navigation, true) );
			\error_log( 'pulsecore/page/admin/manage_navigation.php: Unable to build navigation list END' );
		}
		
		# breadcrumbs
		$GLOBALS['breadcrumb_custom_settings'] = (object)array(
			'entries' => []
		);
		$GLOBALS['breadcrumb_custom_settings']->entries['Manage Navigation'] = PULSE_ADMIN_URL . '/index.php?p=manage_navigation&method=get';
		
		# render
		$view = new \pulsecore\View( \pulsecore\get_configs()->dir_template . '/admin/manage_navigation_list.phtml' );
		
		# vue
		\pulsecore\get_context()->theme->js_body->add( PULSE_BASE_URL . '/pulsecore/asset/vendor/node_modules/vue/dist/vue.js',                       [],      'vue' );
		\pulsecore\get_context()->theme->js_body->add( PULSE_BASE_URL . '/pulsecore/asset/vendor/node_modules/sortablejs/Sortable.min.js',            ['vue'], 'sortable' );
		\pulsecore\get_context()->theme->js_body->add( PULSE_BASE_URL . '/pulsecore/asset/vendor/node_modules/vuedraggable/dist/vuedraggable.umd.js', ['vue',  'sortable'] );
		
		# new version
		\pulsecore\get_context()->theme->css      ->add( PULSE_BASE_URL . '/pulsecore/asset/js/widget/manage_navigation/manage-navigation.css', [] );
		\pulsecore\get_context()->theme->js_module->add( PULSE_BASE_URL . '/pulsecore/asset/js/widget/manage_navigation/manage-navigation.js', [] );
		
		$result = $view->render_captured( $view_model );
		
		return $result;
	}
	
	/**
	 * put request
	 * @param array $request_params array of request parameters
	 * @param array $request_cookie array of cookie parameters
	 * @return string
	 */
	protected function handle_put (array $request_params, array $request_cookie) : string {
		
		\pulsecore\pre_condition(     isset($request_params['active']) );
		\pulsecore\pre_condition( \is_array($request_params['active']) );
		\pulsecore\pre_condition(   \sizeof($request_params['active']) > 0);
		
		\pulsecore\pre_condition(     isset($request_params['nav_list']) );
		\pulsecore\pre_condition( \is_array($request_params['nav_list']) );
		\pulsecore\pre_condition(   \sizeof($request_params['nav_list']) > 0);
		
		\pulsecore\pre_condition(     isset($request_params['nuke']) );
		\pulsecore\pre_condition( \is_array($request_params['nuke']) );
		\pulsecore\pre_condition(   \sizeof($request_params['nuke']) > 0);
		
		# CSRF
		$this->csrf_check( $request_params );
		
		# process data if present
		if (\sizeof($request_params['nav_list']) > 0) {
			
			/*
			# cleanup
			$navigation = [];
			
			foreach ($request_params['nav_list'] as $menu_key => $menu_value) {
				
				$menu_name = \pulsecore\filter\navigation( $menu_key );
				
				# menus
				if (\is_array($menu_value)) {
					
					$navigation[$menu_name] = [];
					
					foreach( $menu_value as $item_key => $item_value) {
						
						# basic menu item
						if (\is_string($item_value)) {
							
							if (\stripos($item_value, '//') !== false) {
								$link = \pulsecore\filter\url( $item_value );
								
							} else {
								$link = \pulsecore\filter\item_url( $item_value );
								
								$link = \ltrim($link, '/');
								$link = '/' . $link;
							}
							
							$active = (isset($request_params['active'][$menu_key][$item_key]) and ($request_params['active'][$menu_key][$item_key] == '1'));
							$nuke   = (isset($request_params['nuke'  ][$menu_key][$item_key]) and ($request_params['nuke'  ][$menu_key][$item_key] == '1'));
							
							if (!$nuke) {
								$navigation[$menu_name][$item_key] = (object)array( 'url' => $link, 'active' => $active);
							}
						}
						
						# sub menu
						if (\is_array($item_value)) {
								
								foreach ($item_value as $sub_menu_name => $sub_menu_value) {
									
									if (\stripos($sub_menu_value, '//') !== false) {
										$link = \pulsecore\filter\url( $sub_menu_value );
										
									} else {
										$link = \pulsecore\filter\item_url( $sub_menu_value );
										
										$link = \ltrim($link, '/');
										$link = '/' . $link;
									}
									
									$active = (isset($request_params['active'][$menu_key][$item_key][$sub_menu_name]) and ($request_params['active'][$menu_key][$item_key][$sub_menu_name] == '1'));
									$nuke   = (isset($request_params['nuke'  ][$menu_key][$item_key][$sub_menu_name]) and ($request_params['nuke'  ][$menu_key][$item_key][$sub_menu_name] == '1'));
									
									if (!$nuke) {
										$navigation[$menu_name][$item_key][$sub_menu_name] = (object)array( 'url' => $link, 'active' => $active);
									}
								}
						}
					}
					
					# corner case - remove empty sub-menus
					if (\sizeof($navigation[$menu_name]) == 0) {
						unset($navigation[$menu_name]);
					}
				}
			}
			
			# corner case - empty navigation array should be saved as an object
			if (\sizeof($navigation) == 0) {
				$navigation = (object)[];
			}
			*/
			
			$navigation = $this->process_menu( 
				$request_params['active'],
				$request_params['nav_list'],
				$request_params['nuke'  ]
			);
			
			# store
			$data = \pulsecore\wedge\config\get_json_configs()->json;
			
			$data->navigation = $navigation;
		
			\pulsecore\wedge\config\save_config( $data );
			
			# status message
			\pulsecore\session\status_add( $GLOBALS['lang_status_ok'] );
			\session_write_close();
			
			# re-render
			$path = PULSE_ADMIN_URL;
			\header("Location: {$path}/index.php?p=manage_navigation");
			exit;
		}
		
		return '';
	}
	
	/**
	 * process menus when setting the navigation
	 * @param array $active_list
	 * @param array $nav_list
	 * @param array $nuke_list
	 * @return mixed
	 */
	protected function process_menu (array $active_list, array $nav_list, array $nuke_list) {
		
		# cleanup
		$navigation = [];
		
		foreach ($nav_list as $menu_key => $menu_value) {
			
			$menu_name = \pulsecore\filter\navigation( $menu_key );
			
			$active = (isset($active_list[$menu_key]) and ($active_list[$menu_key] == '1'));
			$nuke   = (isset($nuke_list[  $menu_key]) and ($nuke_list[  $menu_key] == '1'));
			
			# basic menu item
			if (\is_string($menu_value)) {
				
				if (\stripos($menu_value, '//') !== false) {
					$link = \pulsecore\filter\url( $menu_value );
					
				} else {
					$link = \pulsecore\filter\item_url( $menu_value );
					
					$link = \ltrim($link, '/');
					$link = '/' . $link;
				}
				
				if (!$nuke) {
					$navigation[$menu_name] = (object)[ 'url' => $link, 'active' => $active ];
				}
			}
			
			# sub menu
			if (\is_array($menu_value)) {
				
				if (!$nuke) {
					
					$navigation[$menu_name] = $this->process_menu(
						$active_list[$menu_key],
						$nav_list[   $menu_key],
						$nuke_list[  $menu_key]
					);
				};
			}
			
			# corner case - remove empty sub-menus
			if (\sizeof((array)$navigation[$menu_name]) == 0) {
				unset($navigation[$menu_name]);
			}
			
		}
		
		# corner case - empty navigation array should be saved as an object
		if (\sizeof($navigation) == 0) {
			$navigation = (object)[];
		}
		
		
		return $navigation;
	}
}
