<?php
/* TOP_COMMENT_START
 * Copyright (C) 2022, Levion Partners dba PulseCMS - All Rights Reserved
 *
 * This file is part of Pulse Core. It may be used by individuals or organizations generating less than $400,000 USD per year in revenue, free-of-charge. Individuals or organizations generating over $400,000 in annual revenue who continue to use Pulse Core after 90 days for non-evaluation and non-development use must purchase a paid license. 
 *
 * Proprietary
 * You may modify this source code for internal use. Resale or redistribution is prohibited.
 *
 * You can get the latest version at: https://pulsecms.com/
 *
 * Dated June 2023
 *
TOP_COMMENT_END */

declare(strict_types = 1);

namespace pulsecore\logic;

/**
 * geolocate an IP address
 */
class GeoIpIpApi extends Base {
	
	/**
	 * parse a given IP via http://freegeoip.net/
	 * @param array $arguments array optional list of parameters
	 * @return stdClass
	 */
	public function process (array $arguments = []) {
		
		\pulsecore\pre_condition(      isset($arguments['ip']) );
		\pulsecore\pre_condition( \is_string($arguments['ip']) );
		\pulsecore\pre_condition(    \strlen($arguments['ip']) > 0);
		
		$ip = \trim($arguments['ip']);
		
		$result = new \stdClass();
		$result->country = 'unknown';
		
		$api_key = \pulsecore\wedge\config\get_json_configs()->json->geoip->api_key;
		
		if (true or (\strlen($api_key) > 0)) { # no API key needed
			
			# load data from URL
			$curl_handle = \curl_init();
			
			$url = 'http://ip-api.com/json/' . \urlencode($ip); # . '?access_key=' . \urlencode($api_key);
			
			\curl_setopt_array(
				$curl_handle,
				array(
					\CURLOPT_URL            => $url,
					\CURLOPT_USERAGENT      => 'spider',
					
					\CURLOPT_ENCODING       => 'UTF-8',
					\CURLOPT_RETURNTRANSFER => true,
					
					\CURLOPT_TIMEOUT        => 120,
					\CURLOPT_CONNECTTIMEOUT =>  30,
					
					\CURLOPT_MAXREDIRS      =>   5,
					\CURLOPT_FOLLOWLOCATION => true
				)
			);
			
			$data = \curl_exec($curl_handle);
			
			$decoded = \json_decode( $data );
			
			if (isset($decoded->country)) {
				$result->country = $decoded->country;
			}
			
			\pulsecore\invariant(($result !== false), ('Curl error: ' . \curl_error($curl_handle)) );
			
			\curl_close($curl_handle);
		}
		
		return $result;
	}
}
