<?php
/* TOP_COMMENT_START
 * Copyright (C) 2022, Levion Partners dba PulseCMS - All Rights Reserved
 *
 * This file is part of Pulse Core. It may be used by individuals or organizations generating less than $400,000 USD per year in revenue, free-of-charge. Individuals or organizations generating over $400,000 in annual revenue who continue to use Pulse Core after 90 days for non-evaluation and non-development use must purchase a paid license. 
 *
 * Proprietary
 * You may modify this source code for internal use. Resale or redistribution is prohibited.
 *
 * You can get the latest version at: https://pulsecms.com/
 *
 * Dated June 2023
 *
TOP_COMMENT_END */

namespace pulsecore\store\blog;

/**
 * base class for blog storage backend
 */
abstract class Base extends \pulsecore\store\Base {
	
	/**
	 * list all the blogs in a directory
	 * \param $directory string
	 * \return stdClass containing roll of \pulsecore\store\blog\Roll objects and a default_blog one for the container directory (default blog case)
	 */
	public static function list_blogs( $directory ) {
		
		\pulsecore\pre_condition(      isset($directory) );
		\pulsecore\pre_condition( \is_string($directory) );
		\pulsecore\pre_condition(    \strlen($directory) > 0);
		
		$result = (object)array(
			'default_blog' => new Roll($directory), 
			'roll'         => array(),
		);
		
		$items = \glob( $directory . '/*' );
		
		foreach ($items as $value) {
			
			# blog items - skip
			# if (\is_file($value)) {
			# }
			
			# directory
			if (\is_dir($value)) {
				$result->roll[] = new Roll($value);
			}
		}
		
		return $result;
	}
	
	/*
	 * list all the blogs in a directory and sb directories - only blog Items are generated
	 * \param $directory string
	 * \return array  \pulsecore\store\blog\Item objects 
	 */
	public static function list_blogs_only( $directory ) {
		
		\pulsecore\pre_condition(      isset($directory) );
		\pulsecore\pre_condition( \is_string($directory) );
		\pulsecore\pre_condition(    \strlen($directory) > 0);
		
		$result = array();
		
		$blog_files = Base::list_blogs( $directory );
		
		$working_set   = array();
		$working_set[] = $blog_files->default_blog; 
		$working_set   = \array_merge( $working_set, $blog_files->roll );
		
		while (\sizeof($working_set) > 0) {
			
			$item = \array_shift( $working_set );
			
			if ($item instanceof \pulsecore\store\blog\Roll) {
				$working_set = \array_merge( $working_set, $item->items( 1, $item->size()) );
				$working_set = \array_merge( $working_set, $item->sub_rolls() );
			}
			
			if ($item instanceof \pulsecore\store\blog\Item) {
				$result[] = $item;
			}
		}
		
		return $result;
	}
}
